
#ifdef VERTEX_SHADER
		 #ifdef NEW_GPU
			 layout(location = 0) in vec3 Position;  
			 layout(location = 1) in vec3 Normal;
			 layout(location = 2) in vec3 Ambient;
			 layout(location = 3) in vec3 Diffuse;

		 #else
			 in vec3 Position;  
			 in vec3 Normal;
			 in vec3 Ambient;
			 in vec3 Diffuse;
		 #endif
	
	#ifdef RenderPass_Shaded	
	
		 out vec3 transNormal;  
		 out vec3 lightToVertex; 
		 out vec4 vertex_worldSpace;
		 out vec4 lightPos_eyeSpace;
		 out vec4 pos_eyeSpace;
		 out vec3 ambient_color;
		 out vec3 diffuse_color;

		 uniform mat4 u_mvp;
		 uniform mat4 u_modelMatrix;
		 uniform mat3 u_normalMatrix;
		 uniform mat4 u_viewMatrix;  
		 uniform vec3 u_lightPosition; 
		
		 void main()  
		 {
		 		vertex_worldSpace = u_modelMatrix * vec4(Position, 1.0);					
				pos_eyeSpace = u_viewMatrix * vertex_worldSpace;
				lightPos_eyeSpace = u_viewMatrix * vec4(u_lightPosition, 1.0);
				transNormal = u_normalMatrix * Normal;
				lightToVertex = pos_eyeSpace.xyz - lightPos_eyeSpace.xyz;
				ambient_color = Ambient;
				diffuse_color = Diffuse;
				gl_Position = u_mvp * vec4(Position, 1.0);
		 }
	#endif
	#ifdef RenderPass_Depth
		uniform mat4 u_modelMatrix;
		void main()  
		{
				gl_Position = u_modelMatrix * vec4(Position, 1.0);
		}
	#endif

	#ifdef RenderPass_Env
		uniform mat4 u_modelMatrix;
		out vec3 normal_OS;
		out vec3 ambient_color;
		out vec3 diffuse_color;
		void main()  
		{
				normal_OS = Normal;
				ambient_color = Ambient;
				diffuse_color = Diffuse;
				gl_Position = u_modelMatrix * vec4(Position, 1.0);
		}
	#endif

#endif
#ifdef GEOMETRY_SHADER

		layout(triangles, invocations = 6) in;
		layout(triangle_strip, max_vertices=18) out;

#ifdef RenderPass_Depth		
		uniform mat4 u_lightMatrices[6]; //world to light-clipspace

		out vec4 WS_pos_from_GS;

		void main() 
		{
			gl_Layer = gl_InvocationID;
			for(int tri_vert = 0; tri_vert < 3; ++tri_vert) 
			{
				WS_pos_from_GS = gl_in[tri_vert].gl_Position;
				gl_Position = u_lightMatrices[gl_Layer] * WS_pos_from_GS;
				EmitVertex();
			}
			EndPrimitive();
		}
#endif
#ifdef RenderPass_Env


		uniform mat4 u_lightVPMatrices[6]; //world to light-clipspace
		uniform mat3 u_lightNormalMatrices[6];
		uniform mat4 u_lightViewMatrices[6];
		uniform vec3 u_lightPosition; 

		in vec3 normal_OS[3];
		in vec3 color_ambient[3];
		in vec3 color_diffuse[3];
		
		out vec3 transNormal;
		out vec3 lightToVertex;
		out vec4 vertex_worldSpace;
		out vec4 lightPos_eyeSpace;
		out vec4 pos_eyeSpace;
		out vec3 ambient_color;
		out vec3 diffuse_color;

		void main() 
		{
			gl_Layer = gl_InvocationID;
			for(int tri_vert = 0; tri_vert < 3; ++tri_vert) 
			{
				vertex_worldSpace = gl_in[tri_vert].gl_Position;
				transNormal = u_lightNormalMatrices[gl_Layer] * normal_OS[tri_vert];
				pos_eyeSpace = u_lightViewMatrices[gl_Layer] * gl_in[tri_vert].gl_Position;
				lightPos_eyeSpace = u_lightViewMatrices[gl_Layer] * vec4(u_lightPosition, 1.0);
				lightToVertex = pos_eyeSpace.xyz - lightPos_eyeSpace.xyz;
				ambient_color = color_ambient[tri_vert];
				diffuse_color = color_diffuse[tri_vert];

				gl_Position = u_lightVPMatrices[gl_Layer] * vertex_worldSpace;
				EmitVertex();
			}
			EndPrimitive();
		}

#endif

#endif
#ifdef FRAGMENT_SHADER


		#ifdef RenderPass_Env

			uniform float u_specularTerm;
			uniform vec3 u_lightColor;
			uniform samplerCube u_shadowCube;
			uniform float u_distanceToFar;
			uniform vec3 u_lightPosition; 
			uniform float u_lightIntensity;

			in vec3 transNormal;
			in vec3 lightToVertex;
			in vec4 vertex_worldSpace;
			in vec4 lightPos_eyeSpace;
			in vec4 pos_eyeSpace;
			in vec3 ambient_color;
			in vec3 diffuse_color;

			out vec4 out_Color;


			void main()
			{
					
				vec3 lightToVertex = vertex_worldSpace.xyz - u_lightPosition;
				vec2 moments = texture(u_shadowCube, lightToVertex).rg;

				float distanceFromVertex = length(lightToVertex);

				float shadow = shadowContribution(moments, distanceFromVertex);

				vec3 n = normalize(transNormal);
				vec3 L = normalize(lightPos_eyeSpace.xyz - pos_eyeSpace.xyz);
				float diffuse = max(dot(n, L), 0.0);

				out_Color = (vec4(ambient_color, 1.0) + (diffuse*vec4(diffuse_color, 1.0)) * shadow) * u_lightIntensity;
				
			}

		#endif

		#ifdef RenderPass_Shaded

			uniform vec3 u_lightColor;
			uniform samplerCube u_shadowCube;
			uniform float u_distanceToFar;
			uniform vec3 u_lightPosition; 
			uniform float u_lightIntensity;

			in vec3 transNormal;
			in vec3 lightToVertex;
			in vec4 vertex_worldSpace;
			in vec4 lightPos_eyeSpace;
			in vec4 pos_eyeSpace;
			in vec3 ambient_color;
			in vec3 diffuse_color;

			out vec4 out_Color;

			#ifndef NEW_GPU
				void main()
				{
					
					vec3 lightToVertex = vertex_worldSpace.xyz - u_lightPosition;
					
					float distanceFromVertex = length(lightToVertex);
			
					vec3 n = normalize(transNormal);
					vec3 L = normalize(lightPos_eyeSpace.xyz - pos_eyeSpace.xyz);
					float diffuse = max(dot(n, L), 0.0f);
	
					out_Color = (vec4(ambient_color, 1.0) + (diffuse*vec4(diffuse_color, 1.0))) * u_lightIntensity;							
				}
			#else
				void main()
				{
					
					vec3 lightToVertex = vertex_worldSpace.xyz - u_lightPosition;
					vec2 moments = texture(u_shadowCube, lightToVertex).rg;

					float distanceFromVertex = length(lightToVertex);

					float shadow = shadowContribution(moments, distanceFromVertex);

					vec3 n = normalize(transNormal);
					vec3 L = normalize(lightPos_eyeSpace.xyz - pos_eyeSpace.xyz);
					float diffuse = max(dot(n, L), 0.0);

					out_Color = (vec4(ambient_color, 1.0) + (diffuse*vec4(diffuse_color, 1.0)) * shadow) * u_lightIntensity;
				
				}
			#endif

		#endif
		
		#ifdef RenderPass_Depth

			uniform vec3 u_lightPosition;
			uniform float u_distanceToFar;

			in vec4 WS_pos_from_GS;

			out vec2 out_moments;

			void main()
			{
				float WS_dist = distance(WS_pos_from_GS.xyz, u_lightPosition);
			
				float WS_dist_normalized = WS_dist / u_distanceToFar;
				out_moments = calcMoments(WS_dist);
				gl_FragDepth = WS_dist_normalized;
			}

		#endif

#endif

