#pragma once

#include "../../../LDEngine/Shader.h"
#include "../../../LDEngine/Log.h"

namespace LD
{

class Diffuse : public Shader
{
public:

	enum DiffuseUniforms
	{
		u_mvp = 0,
		u_viewMatrix,
		u_modelMatrix,
		u_lightMatrices,
		u_normalMatrix,
		u_lightPosition,
		u_lightColor,
		u_ambientColor,
		u_diffuseColor,
		u_distanceToFar,
		u_lightIntensity,
#ifdef NEW_GPU
		u_shadowCube,
		u_envCube,
		u_lightVPMatrices,
		u_lightNormalMatrices,
		u_lightViewMatrices,
#endif
		numberOfUniforms //kinda dirty.. this must be the last element of the uniform
	};


	enum DiffuseAttributes
	{
		position = 0,
		normal,
		ambient,
		diffuse,
		numberOfAttributes
	};

	virtual GLint getUniform(int uniform)
	{
		return _uniforms[uniform];
	}
	
	Diffuse(LD_RenderPass passType) : Shader("Diffuse", passType)
	{
		
		std::vector<GLenum> shaderTypes;
		if(_passType == LD_RenderPass::RenderPass_Shaded)
		{
			shaderTypes.push_back(GL_VERTEX_SHADER);
			shaderTypes.push_back(GL_FRAGMENT_SHADER);
		} 
		else if(_passType == LD_RenderPass::RenderPass_Depth)
		{
			shaderTypes.push_back(GL_VERTEX_SHADER);
			shaderTypes.push_back(GL_GEOMETRY_SHADER); 
			shaderTypes.push_back(GL_FRAGMENT_SHADER);
		} 
		else if(_passType == LD_RenderPass::RenderPass_Env)
		{
			shaderTypes.push_back(GL_VERTEX_SHADER);
			shaderTypes.push_back(GL_GEOMETRY_SHADER); 
			shaderTypes.push_back(GL_FRAGMENT_SHADER);
		}
		else
		{
			Log::error("Unknown Pass Type: " + std::to_string(_passType));
			return;
		}

		createShaderfromSource(shaderTypes, passType);
		initUniforms();
	};

private:

	GLint _uniforms [DiffuseUniforms::numberOfUniforms];

	void initUniforms()
	{
		_uniforms[u_mvp] = glGetUniformLocation(_program, "u_mvp");
		_uniforms[u_normalMatrix] = glGetUniformLocation(_program, "u_normalMatrix");
		_uniforms[u_lightPosition] = glGetUniformLocation(_program, "u_lightPosition");
		_uniforms[u_lightColor] = glGetUniformLocation(_program, "u_lightColor");
		_uniforms[u_viewMatrix] = glGetUniformLocation(_program, "u_viewMatrix");
		_uniforms[u_modelMatrix] = glGetUniformLocation(_program, "u_modelMatrix");
		_uniforms[u_lightMatrices] = glGetUniformLocation(_program, "u_lightMatrices");
		_uniforms[u_distanceToFar] = glGetUniformLocation(_program, "u_distanceToFar");
#ifdef NEW_GPU
		_uniforms[u_shadowCube] = glGetUniformLocation(_program, "u_shadowCube");
		_uniforms[u_envCube] = glGetUniformLocation(_program, "u_envCube");
		_uniforms[u_lightVPMatrices] = glGetUniformLocation(_program, "u_lightVPMatrices");
		_uniforms[u_lightNormalMatrices] = glGetUniformLocation(_program, "u_lightNormalMatrices");
		_uniforms[u_lightViewMatrices] = glGetUniformLocation(_program, "u_lightViewMatrices");
#endif
		_uniforms[u_ambientColor] = glGetUniformLocation(_program, "u_ambientColor");
		_uniforms[u_diffuseColor] = glGetUniformLocation(_program, "u_diffuseColor");
		_uniforms[u_lightIntensity] = glGetUniformLocation(_program, "u_lightIntensity");
	}

};

}