
#ifdef VERTEX_SHADER
		#ifdef NEW_GPU
			layout (location = 0) in vec2 Position;
			layout (location = 1) in vec2 TexCoords;
		#else
			in vec2 Position;
			in vec2 TexCoords;
		#endif
	
		uniform mat4 u_mvp;
		uniform mat4 u_modelMatrix;

		out vec2 texC;
	
	#ifdef RenderPass_Shaded
		void main()
		{
			texC = TexCoords;
			gl_Position = u_mvp * vec4(Position, 0.0, 1.0);
		}
	#endif
	#ifdef RenderPass_Env
		void main()
		{
			texC = TexCoords;
			gl_Position = u_modelMatrix * vec4(Position, 0.0, 1.0);
		}
	#endif
#endif

#ifdef GEOMETRY_SHADER

		layout(triangles, invocations = 6) in;
		layout(triangle_strip, max_vertices=18) out;

		uniform mat4 u_lightViewMatrices[6];
		uniform mat4 u_projectionMatrix;
		uniform float u_size;

		in vec2 texC[3];
		out vec2 texCoord[3];

		void main() 
		{
			gl_Layer = gl_InvocationID;
			for(int tri_vert = 0; tri_vert < 3; ++tri_vert) 
			{
				texCoord[tri_vert] = texC[tri_vert];
				mat4 viewMat = u_lightViewMatrices[gl_Layer];
				neutralizeRotation(viewMat, u_size);
				gl_Position = u_projectionMatrix * viewMat * gl_in[tri_vert].gl_Position;
				EmitVertex();
			}
			EndPrimitive();
		}

#endif

#ifdef FRAGMENT_SHADER

	uniform sampler2D u_lightSpotTexture;
	uniform vec3	  u_lightColor;

	in vec2 texCoord;


	out vec4 out_Color;

	void main()
	{
		float alpha = texture(u_lightSpotTexture, texCoord).a;
		out_Color = vec4(u_lightColor, alpha);
	}

#endif


