#pragma once

#include "../../../LDEngine/Shader.h"
#include "../../../LDEngine/Log.h" 


namespace LD
{

class LightShader : public Shader
{
public:

	enum LightUniforms
	{
		u_mvp,
		u_modelMatrix,
		u_lightViewMatrices,
		u_projectionMatrix,
		u_size,
		u_lightSpotTexture,
		u_lightColor,
		numberOfUniforms //kinda dirty.. this must be the last element of the uniform
	};

	enum LightAttributes
	{
		position = 0,
		texCoords,
		numberOfAttributes
	};

	virtual GLint getUniform(int uniform)
	{
		return _uniforms[uniform];
	}
	
	LightShader(LD_RenderPass passType) : Shader("LightShader", passType)
	{
		std::vector<GLenum> shaderTypes;
		if(passType == LD_RenderPass::RenderPass_Shaded)
		{
			shaderTypes.push_back(GL_VERTEX_SHADER);
			shaderTypes.push_back(GL_FRAGMENT_SHADER);		
		}
		if(passType == LD_RenderPass::RenderPass_Env)
		{
			shaderTypes.push_back(GL_VERTEX_SHADER);
			shaderTypes.push_back(GL_GEOMETRY_SHADER);
			shaderTypes.push_back(GL_FRAGMENT_SHADER);
		}

		createShaderfromSource(shaderTypes, _passType);
		initUniforms();
	}


	private:
		GLint _uniforms [LightUniforms::numberOfUniforms];

		void initUniforms()
		{

			_uniforms[u_lightViewMatrices] = glGetUniformLocation(_program, "u_lightViewMatrices");
			_uniforms[u_projectionMatrix] = glGetUniformLocation(_program, "u_projectionMatrix");
			_uniforms[u_size] = glGetUniformLocation(_program, "u_size");
			_uniforms[u_mvp] = glGetUniformLocation(_program, "u_mvp");
			_uniforms[u_lightSpotTexture] = glGetUniformLocation(_program, "u_lightSpotTexture");
			_uniforms[u_modelMatrix] = glGetUniformLocation(_program, "u_modelMatrix");
			_uniforms[u_lightColor] = glGetUniformLocation(_program, "u_lightColor");
		}
		
};


}