#pragma once

#include "LDCommon.h"
#include "../../../LDEngine/Shader.h"
#include "../../../LDEngine/Log.h"

namespace LD
{

	class MotionBlur : public Shader
	{
	public:
		enum MotionBlurTextureUnits {
			COLOR_TEX_UNIT = 0,
			DEPTH_TEX_UNIT,
			TUNNEL_TEX_UNIT
		};

		enum MotionBlurUniforms
		{
			u_numSamples = 0,
			u_invViewProjection,
			u_prevViewProjection,
			u_colorTex,
			u_depthTex,
			u_viewPort,
			u_alphaBlend, // Uniforms needed for the tunnel effect
			u_tunnelTex,
			u_smWidth,
			u_smHeight,
			u_tunnelRot,
			u_tunnelTrans,
			numberOfUniforms //kinda dirty.. this must be the last element of the uniform
		};

		enum PhongAttributes
		{
			position = 0,
			normal,
			numberOfAttributes
		};

		virtual GLint getUniform(int uniform)
		{
			return _uniforms[uniform];
		}

		MotionBlur(LD_RenderPass passType) : Shader("MotionBlur", passType)
		{
			std::vector<GLenum> shaderTypes;
			if(_passType == LD_RenderPass::RenderPass_MotionBlur) {
				shaderTypes.push_back(GL_VERTEX_SHADER);
				shaderTypes.push_back(GL_FRAGMENT_SHADER);
			} else {
				Log::error("Unknown Pass Type: " + std::to_string(_passType));
				return;
			}

			createShaderfromSource(shaderTypes, passType);
			initUniforms();
		};

	private:

		GLint _uniforms [MotionBlurUniforms::numberOfUniforms];

		void initUniforms()
		{
			_uniforms[u_numSamples] = glGetUniformLocation(_program, "u_numSamples");
			_uniforms[u_invViewProjection] = glGetUniformLocation(_program, "u_invViewProjection");
			_uniforms[u_prevViewProjection] = glGetUniformLocation(_program, "u_prevViewProjection");
			_uniforms[u_colorTex] = glGetUniformLocation(_program, "u_colorTex");
			_uniforms[u_depthTex] = glGetUniformLocation(_program, "u_depthTex");
			_uniforms[u_tunnelTex] = glGetUniformLocation(_program, "u_tunnelTex");
			_uniforms[u_viewPort] = glGetUniformLocation(_program, "u_viewPort");
			_uniforms[u_alphaBlend] = glGetUniformLocation(_program, "u_alphaBlend");
			_uniforms[u_smWidth] = glGetUniformLocation(_program, "u_smWidth");
			_uniforms[u_smHeight] = glGetUniformLocation(_program, "u_smHeight");
			_uniforms[u_tunnelRot] = glGetUniformLocation(_program, "u_tunnelRot");
			_uniforms[u_tunnelTrans] = glGetUniformLocation(_program, "u_tunnelTrans");
		}
	};

}