
#ifdef VERTEX_SHADER
		 #ifdef NEW_GPU
			 layout(location = 0) in vec3 Position;  
			 layout(location = 1) in vec3 Normal;
		 #else
			 in vec3 Position;  
			 in vec3 Normal;
		 #endif
	
	#ifdef RenderPass_Shaded	
	
		 out vec3 transNormal;  
		 out vec3 lightToVertex; 
		 out vec4 vertex_worldSpace;
		 out vec4 lightPos_eyeSpace;
		 out vec4 pos_eyeSpace;

		 uniform mat4 u_mvp;
		 uniform mat4 u_modelMatrix;
		 uniform mat3 u_normalMatrix;
		 uniform mat4 u_viewMatrix;  
		 uniform vec3 u_lightPosition; 
		
		 void main()  
		 {
		 		vertex_worldSpace = u_modelMatrix * vec4(Position, 1.0);					
				pos_eyeSpace = u_viewMatrix * vertex_worldSpace;
				lightPos_eyeSpace = u_viewMatrix * vec4(u_lightPosition, 1.0);
				transNormal = u_normalMatrix * Normal;
				lightToVertex = pos_eyeSpace.xyz - lightPos_eyeSpace.xyz;
				gl_Position = u_mvp * vec4(Position, 1.0);
		 }
	#endif
	#ifdef RenderPass_Depth
		uniform mat4 u_modelMatrix;
		void main()  
		{
				gl_Position = u_modelMatrix * vec4(Position, 1.0);
		}
	#endif

	#ifdef RenderPass_Env
		uniform mat4 u_modelMatrix;
		out vec3 normal_OS;
		void main()  
		{
				normal_OS = Normal;
				gl_Position = u_modelMatrix * vec4(Position, 1.0);
		}
	#endif

#endif
#ifdef GEOMETRY_SHADER

		layout(triangles, invocations = 6) in;
		layout(triangle_strip, max_vertices=18) out;

#ifdef RenderPass_Depth		
		uniform mat4 u_lightMatrices[6]; //world to light-clipspace

		out vec4 WS_pos_from_GS;

		void main() 
		{
			gl_Layer = gl_InvocationID;
			for(int tri_vert = 0; tri_vert < 3; ++tri_vert) 
			{
				WS_pos_from_GS = gl_in[tri_vert].gl_Position;
				gl_Position = u_lightMatrices[gl_Layer] * WS_pos_from_GS;
				EmitVertex();
			}
			EndPrimitive();
		}
#endif
#ifdef RenderPass_Env


		uniform mat4 u_lightVPMatrices[6]; //world to light-clipspace
		uniform mat3 u_lightNormalMatrices[6];
		uniform mat4 u_lightViewMatrices[6];
		uniform vec3 u_lightPosition; 

		in vec3 normal_OS[];
		
		out vec3 transNormal;
		out vec3 lightToVertex;
		out vec4 vertex_worldSpace;
		out vec4 lightPos_eyeSpace;
		out vec4 pos_eyeSpace;

		void main() 
		{
			gl_Layer = gl_InvocationID;
			for(int tri_vert = 0; tri_vert < 3; ++tri_vert) 
			{
				vertex_worldSpace = gl_in[tri_vert].gl_Position;
				transNormal = u_lightNormalMatrices[gl_Layer] * normal_OS[tri_vert];
				pos_eyeSpace = u_lightViewMatrices[gl_Layer] * gl_in[tri_vert].gl_Position;
				lightPos_eyeSpace = u_lightViewMatrices[gl_Layer] * vec4(u_lightPosition, 1.0);
				lightToVertex = pos_eyeSpace.xyz - lightPos_eyeSpace.xyz;

				gl_Position = u_lightVPMatrices[gl_Layer] * vertex_worldSpace;
				EmitVertex();
			}
			EndPrimitive();
		}

#endif

#endif
#ifdef FRAGMENT_SHADER


		#ifdef RenderPass_Env

			uniform vec4 u_ambientColor;
			uniform vec4 u_diffuseColor;
			uniform float u_specularTerm;
			uniform vec3 u_lightColor;
			uniform samplerCube u_shadowCube;
			uniform float u_distanceToFar;
			uniform vec3 u_lightPosition; 
			uniform float u_lightIntensity;

			in vec3 transNormal;
			in vec3 lightToVertex;
			in vec4 vertex_worldSpace;
			in vec4 lightPos_eyeSpace;
			in vec4 pos_eyeSpace;

			out vec4 out_Color;


			void main()
			{
					
				vec3 lightToVertex = vertex_worldSpace.xyz - u_lightPosition;
				vec2 moments = texture(u_shadowCube, lightToVertex).rg;
				
				float distanceFromVertex = length(lightToVertex);

				float shadow = shadowContribution(moments, distanceFromVertex);
			
				out_Color = u_ambientColor * u_lightIntensity;

				vec3 n = normalize(transNormal);
				vec3 L = normalize(lightPos_eyeSpace.xyz - pos_eyeSpace.xyz);
				vec3 R = normalize(reflect(L, n));

				float diffuse = max(dot(n, L), 0.0);
				float specular = pow(max(dot(R, normalize(pos_eyeSpace.xyz)), 0.0), u_specularTerm);

				out_Color += (diffuse*u_diffuseColor + specular*u_diffuseColor) * shadow * u_lightIntensity;
				
			}

		#endif

		#ifdef RenderPass_Shaded

			uniform vec4 u_ambientColor;
			uniform vec4 u_diffuseColor;
			uniform vec4 u_specularColor;
			uniform float u_specularTerm;
			uniform vec3 u_lightColor;
			uniform samplerCube u_shadowCube;
			uniform float u_distanceToFar;
			uniform vec3 u_lightPosition;
			uniform float u_lightIntensity;

			in vec3 transNormal;
			in vec3 lightToVertex;
			in vec4 vertex_worldSpace;
			in vec4 lightPos_eyeSpace;
			in vec4 pos_eyeSpace;

			out vec4 out_Color;

			#ifndef NEW_GPU
				void main()
				{
					
					vec3 lightToVertex = vertex_worldSpace.xyz - u_lightPosition;
					
					float distanceFromVertex = length(lightToVertex);
			
					out_Color = u_ambientColor * u_lightIntensity;

					vec3 n = normalize(transNormal);
					vec3 L = normalize(lightPos_eyeSpace.xyz - pos_eyeSpace.xyz);
					vec3 R = normalize(reflect(L, n));

					float diffuse = max(dot(n, L), 0.0f);
					
					float specular = pow(max(dot(R, normalize(pos_eyeSpace.xyz)), 0.0f), u_specularTerm);

					out_Color += (diffuse*u_diffuseColor + specular*u_specularColor) * u_lightIntensity;				
					//out_Color += (diffuse*u_diffuseColor + specular*u_specularColor) / distanceFromVertex;				
				}
			#else
				void main()
				{
					
					vec3 lightToVertex = vertex_worldSpace.xyz - u_lightPosition;
					vec2 moments = texture(u_shadowCube, lightToVertex).rg;

					float distanceFromVertex = length(lightToVertex);

					float shadow = shadowContribution(moments, distanceFromVertex);
					
					out_Color = u_ambientColor * u_lightIntensity;

					vec3 n = normalize(transNormal);
					vec3 L = normalize(lightPos_eyeSpace.xyz - pos_eyeSpace.xyz);
					vec3 R = normalize(reflect(L, n));

					float diffuse = max(dot(n, L), 0.0);
					
					float specular = pow(max(dot(R, normalize(pos_eyeSpace.xyz)), 0.0f), u_specularTerm);

					out_Color += (diffuse*u_diffuseColor + specular*u_specularColor) * shadow * u_lightIntensity;
				
				}
			#endif

		#endif
		
		#ifdef RenderPass_Depth

			uniform vec3 u_lightPosition;
			uniform float u_distanceToFar;

			in vec4 WS_pos_from_GS;

			out vec2 out_moments;

			void main()
			{
				float WS_dist = distance(WS_pos_from_GS.xyz, u_lightPosition);
			
				float WS_dist_normalized = WS_dist / u_distanceToFar;
				out_moments = calcMoments(WS_dist);
				gl_FragDepth = WS_dist_normalized;
			}

		#endif

#endif

