
#ifdef VERTEX_SHADER

		#ifndef NEW_GPU
			in vec3 Position;  
			in vec3 Normal;  
		#else
			layout(location = 0) in vec3 Position;  
			layout(location = 1) in vec3 Normal;  
		#endif
	
	#ifdef RenderPass_Shaded
		#ifndef NEW_GPU
	
			out vec3 transNormal;  
			out vec3 lightToVertex; 
			out vec4 vertex_worldSpace;
			out vec4 lightPos_eyeSpace;
			out vec4 pos_eyeSpace;

			 uniform mat4 u_mvp;
			 uniform mat4 u_modelMatrix;
			 uniform mat3 u_normalMatrix;
			 uniform mat4 u_viewMatrix;  
			 uniform vec3 u_lightPosition; 
		
			 void main()  
			 {
		 			vertex_worldSpace = u_modelMatrix * vec4(Position, 1.0);					
					pos_eyeSpace = u_viewMatrix * vertex_worldSpace;
					lightPos_eyeSpace = u_viewMatrix * vec4(u_lightPosition, 1.0);
					transNormal = u_normalMatrix * Normal;
					lightToVertex = pos_eyeSpace.xyz - lightPos_eyeSpace.xyz;
					gl_Position = u_mvp * vec4(Position, 1.0);

			 }
		#else
			
			layout(location = 0) out vec3 vertexNormal;
			layout(location = 1) out float visibleDistance;
			
			uniform mat3 u_normalMatrix;
			uniform mat4 u_modelViewMatrix;
			uniform float u_tanFOV;
			uniform float u_aspectRatio;
			uniform float u_near;
			uniform float u_far;

			float culledDistance(vec4 eyeS_Vertex, vec3 eyeS_normal)
			{	
				//backfaces
				if(dot(eyeS_normal, eyeS_Vertex.xyz) >= 0.0)
					return 0.0;
				
				//frustum culling		
				float zProjection = dot(eyeS_Vertex.xyz, vec3(0.0, 0.0, -1.0));
				if(zProjection < u_near || zProjection > u_far)
					return 0.0;

				float boundary = zProjection * u_tanFOV;
				float yProjection = dot(eyeS_Vertex.xyz, vec3(0.0, 1.0, 0.0));
				if(yProjection < - boundary || yProjection > boundary)
					return 0.0;
				
				boundary *= u_aspectRatio;
				float xProjection = dot(eyeS_Vertex.xyz, vec3(1.0, 0.0, 0.0));
				if(xProjection < - boundary || xProjection > boundary)
					return 0.0;

				return length(eyeS_Vertex.xyz);
			}

			void main()
			{
				vertexNormal = u_normalMatrix * Normal;
				vec4 vertex_eyeS = u_modelViewMatrix * vec4(Position, 1.0);
				visibleDistance = culledDistance(vertex_eyeS, vertexNormal);
				gl_Position = vertex_eyeS;
			}

		#endif
	
	#endif
	
	#ifdef RenderPass_Depth
		uniform mat4 u_modelMatrix;
		void main()  
		{
				gl_Position = u_modelMatrix * vec4(Position, 1.0);
		}
	#endif

	#ifdef RenderPass_Env
		uniform mat4 u_modelMatrix;
		out vec3 normal_OS;
		void main()  
		{
				normal_OS = Normal;
				gl_Position = u_modelMatrix * vec4(Position, 1.0);
		}
	#endif

#endif
#ifdef GEOMETRY_SHADER

		layout(triangles, invocations = 6) in;
		layout(triangle_strip, max_vertices=18) out;

#ifdef RenderPass_Depth		
		uniform mat4 u_lightMatrices[6]; //world to light-clipspace

		out vec4 WS_pos_from_GS;

		void main() 
		{
			gl_Layer = gl_InvocationID;
			for(int tri_vert = 0; tri_vert < 3; ++tri_vert) 
			{
				WS_pos_from_GS = gl_in[tri_vert].gl_Position;
				gl_Position = u_lightMatrices[gl_Layer] * WS_pos_from_GS;
				EmitVertex();
			}
			EndPrimitive();

		}
#endif
#ifdef RenderPass_Env


		uniform mat4 u_lightVPMatrices[6]; //world to light-clipspace
		uniform mat3 u_lightNormalMatrices[6];
		uniform mat4 u_lightViewMatrices[6];
		uniform vec3 u_lightPosition; 

		in vec3 normal_OS[];
		
		layout(location = 0) out vec3 transNormal;  
		layout(location = 1) out vec3 lightToVertex; 
		layout(location = 2) out vec4 vertex_worldSpace;
		layout(location = 3) out vec4 lightPos_eyeSpace;
		layout(location = 4) out vec4 pos_eyeSpace;

		void main() 
		{
			gl_Layer = gl_InvocationID;
			for(int tri_vert = 0; tri_vert < 3; ++tri_vert) 
			{
				vertex_worldSpace = gl_in[tri_vert].gl_Position;
				transNormal = u_lightNormalMatrices[gl_Layer] * normal_OS[tri_vert];
				pos_eyeSpace = u_lightViewMatrices[gl_Layer] * gl_in[tri_vert].gl_Position;
				lightPos_eyeSpace = u_lightViewMatrices[gl_Layer] * vec4(u_lightPosition, 1.0);
				lightToVertex = pos_eyeSpace.xyz - lightPos_eyeSpace.xyz;

				gl_Position = u_lightVPMatrices[gl_Layer] * vertex_worldSpace;
				EmitVertex();
			}
			EndPrimitive();
		}

#endif

#endif

#ifdef TESS_CONTROL_SHADER

	uniform float u_TessellationLevel;

	layout(vertices = 3) out;
	
	layout(location = 0) in vec3 vertexNormal[3];
	layout(location = 1) in float visibleDistance[3];

	layout(location = 0) out vec3 passedNormal[3];

	layout(location = 3) out float projUV[3];
	layout(location = 6) out float projVW[3];
	layout(location = 9) out float projWU[3];

	float tessLevel(float camToVertexDistance[3])
	{
		float dSum = camToVertexDistance[0] + camToVertexDistance[1] + camToVertexDistance[2];
		if(dSum <= 0.001)
			return 0.0;

		float minDistance = min(min(camToVertexDistance[0], camToVertexDistance[1]), camToVertexDistance[2]);
		if(minDistance <= 10.0)
			return u_TessellationLevel;
		else
			return u_TessellationLevel * 0.5;
	}

	float buildEdgeProjections(int v1Idx, int v2Idx)
	{
		vec3 v1 = gl_in[v1Idx].gl_Position.xyz;
		float Pi1P2 = v1[gl_InvocationID] - dot(v1 - gl_in[v2Idx].gl_Position.xyz, vertexNormal[v2Idx]) * 
					vertexNormal[v2Idx][gl_InvocationID];

		vec3 v2 = gl_in[v2Idx].gl_Position.xyz;
		float Pi2P1 = v2[gl_InvocationID] - dot(v2 - gl_in[v1Idx].gl_Position.xyz, vertexNormal[v1Idx]) * 
					vertexNormal[v1Idx][gl_InvocationID];

		return Pi1P2 + Pi2P1;
	}

	void main()
	{
		// compute terms for projection
		// compute one coordinate in each invocation
		projUV[gl_InvocationID] = buildEdgeProjections(0,1);
		projVW[gl_InvocationID] = buildEdgeProjections(1,2);
		projWU[gl_InvocationID] = buildEdgeProjections(0,2);

		// calculate tessellation levels
		gl_TessLevelInner[0] = tessLevel(visibleDistance);
		gl_TessLevelOuter[gl_InvocationID] = tessLevel(visibleDistance);
		
		// pass data
		gl_out[gl_InvocationID].gl_Position = gl_in[gl_InvocationID].gl_Position;
		passedNormal[gl_InvocationID] = vertexNormal[gl_InvocationID];
	}

#endif

#ifdef TESS_EVALUATION_SHADER

	layout(triangles, fractional_odd_spacing, ccw) in;

	layout(location = 0) in vec3 passedNormal[3];

	layout(location = 3) in float projUV[3];
	layout(location = 6) in float projVW[3];
	layout(location = 9) in float projWU[3];

	out vec3 transNormal;  
	out vec3 lightToVertex; 
	out vec4 vertex_worldSpace;
	out vec3 lightPos_eyeSpace;
	out vec4 pos_eyeSpace;

	uniform mat4 u_invViewMatrix;
	uniform mat4 u_projectionMatrix;  
	uniform vec3 u_lightPosEyeSpace;
	uniform float u_TessellationFactor;

	void main()
	{
		//barycentric coordinates:
		vec3 uvw = gl_TessCoord;
		vec3 uvw2 = gl_TessCoord * gl_TessCoord;

		// interpolated position (flat)
		vec3 flatTrianglePos = uvw[0] * gl_in[0].gl_Position.xyz 
							 + uvw[1] * gl_in[1].gl_Position.xyz
							 + uvw[2] * gl_in[2].gl_Position.xyz;

		//can't pass vec3, has to be float[3], so here the conversion:
		vec3 vecUV = vec3(projUV[0], projUV[1], projUV[2]);
		vec3 vecVW = vec3(projVW[0], projVW[1], projVW[2]);
		vec3 vecWU = vec3(projWU[0], projWU[1], projWU[2]);

		//barycentric interpolation of projected points:
		vec3 phongPosition = uvw2[0]*gl_in[0].gl_Position.xyz + uvw2[1]*gl_in[1].gl_Position.xyz + uvw2[2]*gl_in[2].gl_Position.xyz
						   + uvw[0]*uvw[1]*vecUV + uvw[1]*uvw[2]*vecVW	+ uvw[2]*uvw[0]*vecWU;

		//normal: barycentric interpolation:
		transNormal = uvw[0]*passedNormal[0] + uvw[1]*passedNormal[1] + uvw[2]*passedNormal[2];
		
		//interpolate between flat and phong-position
		vec3 tessVertexPosition = (1.0 - u_TessellationFactor) * flatTrianglePos + u_TessellationFactor * phongPosition;
		
		vertex_worldSpace = u_invViewMatrix * vec4(tessVertexPosition, 1.0);					
		pos_eyeSpace = vec4(tessVertexPosition, 1.0);
		transNormal = transNormal;
		lightToVertex = tessVertexPosition - u_lightPosEyeSpace;
		lightPos_eyeSpace = u_lightPosEyeSpace;
		gl_Position = u_projectionMatrix * vec4(tessVertexPosition, 1.0);

	}


#endif

#ifdef FRAGMENT_SHADER


		#ifdef RenderPass_Env

			uniform vec4 u_ambientColor;
			uniform vec4 u_diffuseColor;
			uniform float u_specularTerm;
			uniform vec3 u_lightColor;
			uniform samplerCube u_shadowCube;
			uniform float u_distanceToFar;
			uniform vec3 u_lightPosition; 
			uniform float u_lightIntensity;

			in vec3 transNormal;  
			in vec3 lightToVertex; 
			in vec4 vertex_worldSpace;
			in vec4 lightPos_eyeSpace;
			in vec4 pos_eyeSpace;

			layout(location = 0) out vec4 out_Color;


			void main()
			{
					
				vec3 lightToVertex = vertex_worldSpace.xyz - u_lightPosition;
				vec2 moments = texture(u_shadowCube, lightToVertex).rg;

				float distanceFromVertex = length(lightToVertex);

				float shadow = shadowContribution(moments, distanceFromVertex);
			
				out_Color = u_ambientColor * u_lightIntensity;

				vec3 n = normalize(transNormal);
				vec3 L = normalize(lightPos_eyeSpace.xyz - pos_eyeSpace.xyz);
				vec3 R = normalize(reflect(L, n));

				float diffuse = max(dot(n, L), 0.0);
				float specular = pow(max(dot(R, normalize(pos_eyeSpace.xyz)), 0.0), u_specularTerm);

				out_Color += (diffuse*u_diffuseColor + specular*u_diffuseColor) * shadow * u_lightIntensity;
			}

		#endif

		#ifdef RenderPass_Shaded

			uniform vec4 u_ambientColor;
			uniform vec4 u_diffuseColor;
			uniform vec4 u_specularColor;
			uniform float u_specularTerm;
			uniform vec3 u_lightColor;
			#ifdef NEW_GPU
			uniform samplerCube u_shadowCube;
			#endif
			uniform float u_distanceToFar;
			uniform vec3 u_lightPosition; 
			uniform float u_lightIntensity;

			in vec3 transNormal;  
			in vec3 lightToVertex; 
			in vec4 vertex_worldSpace;
			in vec4 lightPos_eyeSpace;
			in vec4 pos_eyeSpace;

			out vec4 out_Color;


			#ifndef NEW_GPU
				void main()
				{
					
					vec3 lightToVertex = vertex_worldSpace.xyz - u_lightPosition;

					float distanceFromVertex = length(lightToVertex);

					out_Color = u_ambientColor * u_lightIntensity;

					vec3 n = normalize(transNormal);
					vec3 L = normalize(lightPos_eyeSpace.xyz - pos_eyeSpace.xyz);
					vec3 R = normalize(reflect(L, n));

					float diffuse = max(dot(n, L), 0.0);
					float specular = pow(max(dot(R, normalize(pos_eyeSpace.xyz)), 0.0), u_specularTerm);

					out_Color += (diffuse*u_diffuseColor + specular*u_specularColor) * u_lightIntensity;

				}
			#else
				void main()
				{
					
					vec3 lightToVertex = vertex_worldSpace.xyz - u_lightPosition;
					vec2 moments = texture(u_shadowCube, lightToVertex).rg;

					float distanceFromVertex = length(lightToVertex);

					float shadow = shadowContribution(moments, distanceFromVertex);
					
					out_Color = u_ambientColor * u_lightIntensity;

					vec3 n = normalize(transNormal);
					vec3 L = normalize(lightPos_eyeSpace.xyz - pos_eyeSpace.xyz);
					vec3 R = normalize(reflect(L, n));

					float diffuse = max(dot(n, L), 0.0);
					float specular = pow(max(dot(R, normalize(pos_eyeSpace.xyz)), 0.0), u_specularTerm);

					out_Color += (diffuse*u_diffuseColor + specular*u_specularColor) * shadow * u_lightIntensity;

				}
			#endif

		#endif
		
		#ifdef RenderPass_Depth

			uniform vec3 u_lightPosition;
			uniform float u_distanceToFar;

			in vec4 WS_pos_from_GS;

			out vec2 out_moments;

			void main()
			{
				float WS_dist = distance(WS_pos_from_GS.xyz, u_lightPosition);
			
				float WS_dist_normalized = WS_dist / u_distanceToFar;
				out_moments = calcMoments(WS_dist);
				gl_FragDepth = WS_dist_normalized;
			}

		#endif

#endif

