#pragma once

#include "../../../LDEngine/Shader.h"
#include "../../../LDEngine/Log.h"

namespace LD
{

class RailShader : public Shader
{
public:

	enum RailUniforms
	{
		u_mvp = 0,
		u_viewMatrix,
		u_modelMatrix,
		u_lightMatrices,
		u_normalMatrix,
		u_lightPosition,
		u_lightPosEyeSpace,
		u_lightColor,
		u_lightIntensity,
		u_ambientColor,
		u_diffuseColor,
		u_specularColor,
		u_specularTerm,
		u_distanceToFar,
#ifdef NEW_GPU
		u_invViewMatrix,
		u_projectionMatrix,  
		u_shadowCube,
		u_envCube,
		u_lightVPMatrices,
		u_lightNormalMatrices,
		u_lightViewMatrices,
		u_TessellationFactor,
		u_TessellationLevel,
		
		//culling uniforms
		u_modelViewMatrix,
		u_tanFOV,
		u_aspectRatio,
		u_far,
		u_near,
#endif
		numberOfUniforms //kinda dirty.. this must be the last element of the uniform
	};


	enum RailAttributes
	{
		position = 0,
		normal,
		numberOfAttributes
	};

	virtual GLint getUniform(int uniform)
	{
		return _uniforms[uniform];
	}
	
	RailShader(LD_RenderPass passType) : Shader("RailShader", passType)
	{
		
		std::vector<GLenum> shaderTypes;
		if(_passType == LD_RenderPass::RenderPass_Shaded)
		{
			shaderTypes.push_back(GL_VERTEX_SHADER);
#ifdef NEW_GPU
			shaderTypes.push_back(GL_TESS_CONTROL_SHADER);
			shaderTypes.push_back(GL_TESS_EVALUATION_SHADER);
#endif
			shaderTypes.push_back(GL_FRAGMENT_SHADER);
		} 
		else if(_passType == LD_RenderPass::RenderPass_Depth)
		{
			shaderTypes.push_back(GL_VERTEX_SHADER);
			shaderTypes.push_back(GL_GEOMETRY_SHADER); 
			shaderTypes.push_back(GL_FRAGMENT_SHADER);
		} 
		else if(_passType == LD_RenderPass::RenderPass_Env)
		{
			shaderTypes.push_back(GL_VERTEX_SHADER);
			shaderTypes.push_back(GL_GEOMETRY_SHADER); 
			shaderTypes.push_back(GL_FRAGMENT_SHADER);
		}
		else
		{
			Log::error("Unknown Pass Type: " + std::to_string(_passType));
			return;
		}

		createShaderfromSource(shaderTypes, passType);
		initUniforms();
	};

private:

	GLint _uniforms [RailUniforms::numberOfUniforms];

	virtual void initUniforms()
	{
		_uniforms[u_mvp] = glGetUniformLocation(_program, "u_mvp");
		_uniforms[u_normalMatrix] = glGetUniformLocation(_program, "u_normalMatrix");
		_uniforms[u_lightPosition] = glGetUniformLocation(_program, "u_lightPosition");
		_uniforms[u_lightColor] = glGetUniformLocation(_program, "u_lightColor");
		_uniforms[u_lightIntensity] = glGetUniformLocation(_program, "u_lightIntensity");
		_uniforms[u_viewMatrix] = glGetUniformLocation(_program, "u_viewMatrix");
		_uniforms[u_modelMatrix] = glGetUniformLocation(_program, "u_modelMatrix");
		_uniforms[u_lightMatrices] = glGetUniformLocation(_program, "u_lightMatrices");
		_uniforms[u_distanceToFar] = glGetUniformLocation(_program, "u_distanceToFar");
		_uniforms[u_lightPosEyeSpace] = glGetUniformLocation(_program, "u_lightPosEyeSpace");
#ifdef NEW_GPU
		_uniforms[u_shadowCube] = glGetUniformLocation(_program, "u_shadowCube");
		_uniforms[u_envCube] = glGetUniformLocation(_program, "u_envCube");
		_uniforms[u_lightVPMatrices] = glGetUniformLocation(_program, "u_lightVPMatrices");
		_uniforms[u_lightNormalMatrices] = glGetUniformLocation(_program, "u_lightNormalMatrices");
		_uniforms[u_lightViewMatrices] = glGetUniformLocation(_program, "u_lightViewMatrices");
		_uniforms[u_TessellationFactor] = glGetUniformLocation(_program, "u_TessellationFactor");
		_uniforms[u_TessellationLevel] = glGetUniformLocation(_program, "u_TessellationLevel");
		_uniforms[u_invViewMatrix] = glGetUniformLocation(_program, "u_invViewMatrix");
		_uniforms[u_projectionMatrix] = glGetUniformLocation(_program, "u_projectionMatrix");
		_uniforms[u_modelViewMatrix] = glGetUniformLocation(_program, "u_modelViewMatrix");
		_uniforms[u_tanFOV] = glGetUniformLocation(_program, "u_tanFOV");
		_uniforms[u_aspectRatio] = glGetUniformLocation(_program, "u_aspectRatio");
		_uniforms[u_far] = glGetUniformLocation(_program, "u_far");
		_uniforms[u_near] = glGetUniformLocation(_program, "u_near");
#endif
		_uniforms[u_ambientColor] = glGetUniformLocation(_program, "u_ambientColor");
		_uniforms[u_diffuseColor] = glGetUniformLocation(_program, "u_diffuseColor");
		_uniforms[u_specularColor] = glGetUniformLocation(_program, "u_specularColor");
		_uniforms[u_specularTerm] = glGetUniformLocation(_program, "u_specularTerm");

		GLenum error = glGetError();
		if (error != GL_NO_ERROR)
		{
			Log::error("Error initializing uniforms of RailShader\n" + oglErrorString(error));
		}

	}

};

}