#pragma once

#include "../../../LDEngine/Shader.h"
#include "../../../LDEngine/Log.h" 


namespace LD
{

class SphereS : public Shader
{
public:

	enum SphereUniforms
	{
		u_modelViewMat = 0,
		u_viewMat,
		u_invViewMat,
		u_projectionMat,
		u_radius,
		u_lightPosition,
		u_lightIntensity,
		u_albedoTexture,
		u_cubeMatTexture,
		u_distanceToFar,
		u_lightMatrices,
		u_shadowCube,
		u_ambientColor,
		u_specularTerm,
		u_lookAt,
		u_modelMatrix,
		u_sphereCenterWS,
		numberOfUniforms //kinda dirty.. this must be the last element of the uniform
	};

	enum SphereAttributes
	{
		position = 0,
		numberOfAttributes
	};

	virtual GLint getUniform(int uniform)
	{
		return _uniforms[uniform];
	}
	
	SphereS(LD_RenderPass passType) : Shader("SphereS", passType)
	{
		std::vector<GLenum> shaderTypes;
		if(_passType == LD_RenderPass::RenderPass_Shaded)
		{
			shaderTypes.push_back(GL_VERTEX_SHADER);
			shaderTypes.push_back(GL_FRAGMENT_SHADER);
		} 
		else if(_passType == LD_RenderPass::RenderPass_Depth)
		{
			shaderTypes.push_back(GL_VERTEX_SHADER);
			shaderTypes.push_back(GL_GEOMETRY_SHADER); 
			shaderTypes.push_back(GL_FRAGMENT_SHADER);
		} 
		else
		{
			Log::error("Unknown Pass Type: " + std::to_string(_passType));
			return;
		}
		
		createShaderfromSource(shaderTypes, _passType);

		initUniforms();
	}


	private:
		GLint _uniforms [SphereUniforms::numberOfUniforms];


		void initUniforms()
		{
			_uniforms[u_radius] = glGetUniformLocation(_program, "u_radius");
			_uniforms[u_projectionMat] = glGetUniformLocation(_program, "u_projectionMat");
			_uniforms[u_lightPosition] = glGetUniformLocation(_program, "u_lightPosition");
			_uniforms[u_albedoTexture] = glGetUniformLocation(_program, "u_albedoTexture");
			_uniforms[u_cubeMatTexture] = glGetUniformLocation(_program, "u_cubeMatTexture");
			_uniforms[u_modelViewMat] = glGetUniformLocation(_program, "u_modelViewMat");
			_uniforms[u_viewMat] = glGetUniformLocation(_program, "u_viewMat");
			_uniforms[u_distanceToFar] = glGetUniformLocation(_program, "u_distanceToFar");	
			_uniforms[u_lightMatrices] = glGetUniformLocation(_program, "u_lightMatrices");	
			_uniforms[u_shadowCube] = glGetUniformLocation(_program, "u_shadowCube");	
			_uniforms[u_invViewMat] = glGetUniformLocation(_program, "u_invViewMat");	
			_uniforms[u_ambientColor] = glGetUniformLocation(_program, "u_ambientColor");
			_uniforms[u_specularTerm] = glGetUniformLocation(_program, "u_specularTerm");
			_uniforms[u_lookAt] = glGetUniformLocation(_program, "u_lookAt");
			_uniforms[u_modelMatrix] = glGetUniformLocation(_program, "u_modelMatrix");
			_uniforms[u_sphereCenterWS] = glGetUniformLocation(_program, "u_sphereCenterWS");
			_uniforms[u_lightIntensity] = glGetUniformLocation(_program, "u_lightIntensity");
		}
		
};


}
