#pragma once

#include "../../../LDEngine/Shader.h"
#include "../../../LDEngine/Log.h" 


namespace LD
{

class SphereText : public Shader
{
public:

	enum SphereTextUniforms
	{
		u_mvp = 0,
		u_textTexture,
		numberOfUniforms //kinda dirty.. this must be the last element of the uniform
	};

	enum SphereTextAttributes
	{
		position = 0,
		texCoords,
		numberOfAttributes
	};

	virtual GLint getUniform(int uniform)
	{
		return _uniforms[uniform];
	}
	
	SphereText(LD_RenderPass passType) : Shader("SphereText", passType)
	{
		std::vector<GLenum> shaderTypes;
		if(_passType == LD_RenderPass::RenderPass_Shaded)
		{
			shaderTypes.push_back(GL_VERTEX_SHADER);
			shaderTypes.push_back(GL_FRAGMENT_SHADER);
		} 
		else if(_passType == LD_RenderPass::RenderPass_Depth)
		{
			shaderTypes.push_back(GL_VERTEX_SHADER);
			shaderTypes.push_back(GL_GEOMETRY_SHADER); 
			shaderTypes.push_back(GL_FRAGMENT_SHADER);
		} 
		else
		{
			Log::error("Unknown Pass Type: " + std::to_string(_passType));
			return;
		}
		
		createShaderfromSource(shaderTypes, _passType);

		initUniforms();
	}


	private:
		GLint _uniforms [SphereTextUniforms::numberOfUniforms];


		void initUniforms()
		{
			_uniforms[u_mvp] = glGetUniformLocation(_program, "u_mvp");
			_uniforms[u_textTexture] = glGetUniformLocation(_program, "u_textTexture");
		}
		
};


}
